import logging
import json
from pathlib import Path

import jsonschema

def get_json(filename: str):
    with open(filename) as f:
        j = json.load(f)
    return j


logging.basicConfig(level=logging.DEBUG)
if __name__ == "__main__":
    targets = get_json("testing/json_compile_targets.json")

    resolver=None
    for target in targets:
        logging.info(f"Testing {target['coreSchema']}")
        schema = get_json(target['coreSchema'])

        supporting_schemas = {schema['$id'] : schema}
        for supporting_schema in target['supportingSchemas']:
            js = get_json(supporting_schema)
            supporting_schemas[js['$id']] = js
        if len(supporting_schemas.keys()) > 0:
            ref_resolver = jsonschema.RefResolver("", "", supporting_schemas)
            print(supporting_schemas.keys())
        else:
            ref_resolver = None

        # validator = jsonschema.Draft202012Validator(schema, resolver=resolver)

        for example_dir in target['exampleFiles']:
            for example_file in Path(example_dir).rglob("*.json"):
                instance = get_json(example_file)
                jsonschema.validate(instance, schema, resolver = ref_resolver)
                # for err in validator.iter_errors(instance):
                #     print(err.message)
        logging.info("Done testing")


# print(json.dumps(js, indent=2))